//+------------------------------------------------------------------+
//|                                           Decode_MarketWatch.mqh |
//|                                            Copyright 2010, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| #include <Decode_MarketWatch.mqh>                                |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| The class for encoding/decoding of Market Watch symbols          |
//+------------------------------------------------------------------+
class CDecodeMarketWatch
  {
   bool              flag;
   bool              ReadFile(string folder_);
public:
   string            symbol[];
   int               symbol_total;
                     CDecodeMarketWatch(void){flag=false;symbol_total=0;};
                    ~CDecodeMarketWatch(void){};
   void              Inits(bool flag_=false);// true- if only the symbols from Market Watch are selected
   bool              InitsTester(string folder_,bool flag_=false)
     {
      // if the function is called in the strategy tester, read the file with symbols, otherwise get symbols from Market Watch
      if((bool)MQL5InfoInteger(MQL5_TESTING)){return(ReadFile(folder_));}
      else return(false);
     };
   void              PrintFile(string folder_);
   //--- encode the symbols as an index number in Market Watch
   string            CodeSymbol(string sym);
   //--- decode the symbol 
   string            DecodeSymbol(string num);
   //--- encode the array of symbols
   string            CodeSymbolArray(const string &sym[]);
   //--- decode the contents of text in the receiving array sym[]
   void              DecodeSymbolArray(string text,string &sym[]);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| initialization of the object, getting the list of symbols        |
//+------------------------------------------------------------------+
void CDecodeMarketWatch::Inits(bool flag_=false)
  {   
   flag=flag_;
   symbol_total=SymbolsTotal(flag);
   ArrayResize(symbol,symbol_total);
   for(int i=0;i<symbol_total;i++)
      symbol[i]=SymbolName(i,flag);

  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| encoding the symbol using two digits, transformation into string |
//+------------------------------------------------------------------+
string CDecodeMarketWatch::CodeSymbol(string sym_)
  {
   string res="",sym=sym_; StringToUpper(sym);
   for(int i=0;i<symbol_total;i++)
     {
      if(sym==symbol[i])
        {
         if(i<10)res="0"+(string)i;
         else res=(string)i;
         break;
        }

     }
   return(res);
  }
//+------------------------------------------------------------------+
//| decoding of two-digit code into a symbol                         |
//+------------------------------------------------------------------+
string CDecodeMarketWatch::DecodeSymbol(string num)
  {
   string res="";int ind=(int)StringToInteger(num);
   if(ind>=0 && ind<symbol_total)res=symbol[ind];
   return(res);
  }
//+------------------------------------------------------------------+
//| getting encoded string from the array of symbols                 |
//+------------------------------------------------------------------+
string CDecodeMarketWatch::CodeSymbolArray(const string &sym[])
  {
   string res="";
   int counts=ArraySize(sym);
   for(int i=0;i<counts;i++)
      res=res+CodeSymbol(sym[i]);
   return(res);
  }
//+------------------------------------------------------------------+
//| getting array of symbols from encoded string                     |
//+------------------------------------------------------------------+
void CDecodeMarketWatch::DecodeSymbolArray(string text,string &sym[])
  {
   int len=StringLen(text)/2; ArrayResize(sym,len);
   for(int i=0;i<len;i++)
      sym[i]=DecodeSymbol(StringSubstr(text,i*2,2));
  }
//+------------------------------------------------------------------+
//| printing of the list of symbols                                  |
//+------------------------------------------------------------------+
void CDecodeMarketWatch::PrintFile(string folder_)
  {
   string folder=folder_;
   int han=FileOpen(folder+".csv",FILE_WRITE|FILE_CSV|FILE_ANSI,";");
   if(han!=INVALID_HANDLE)
     {
      for(int i=0;i<symbol_total;i++)
        {
         FileWrite(han,symbol[i]);
        }

      FileClose(han);
     }
   else Print("File open failed "+folder+".csv, error",GetLastError());
  }
//+------------------------------------------------------------------+
//| reading the list of symbols                                      |
//+------------------------------------------------------------------+
bool CDecodeMarketWatch::ReadFile(string folder_)
  {
   string folder=folder_;
   int han=FileOpen(folder+".csv",FILE_READ|FILE_CSV|FILE_ANSI,";");
   if(han!=INVALID_HANDLE)
     {
      symbol_total=0; ArrayResize(symbol,symbol_total);
      while(!IsStopped())
        {
         string temp=FileReadString(han);
         StringTrimLeft(temp);StringTrimRight(temp);// delete spaces and line feed characters
         StringToUpper(temp);  // make upper case
         //---
         if(temp!="")
           {
            symbol_total++;
            ArrayResize(symbol,symbol_total);
            symbol[symbol_total-1]=temp;
           }
         //---         
         if(FileTell(han)>=FileSize(han))break;
        }
      FileClose(han);
      return(true);
     }
   else
     {
      Print("File open failed "+folder+".csv, error",GetLastError());
      return(false);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
